#ifndef lint
static char sccsid[] = "$Id: read_segy_trace.c,v 1.3 1999/04/07 17:52:31 doug Exp $ ";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/file.h>

#include    "qlib2.h"

#include    "segy.h"
#include    "segy_reel.h"
#include    "procs.h"
#include    "externals.h"

/************************************************************************/
/*  read_segy_trace:							*/
/*	Read SEGY trace header and trace data.				*/
/*  return:								*/
/*	1 on success, 0 on failure.					*/
/*	Set *phdr to point to DATA_HDR.					*/
/*	Set **pdata to point to allocated data buffer.			*/
/*	Caller must free DATA_HDR and data buffer.			*/
/************************************************************************/
int read_segy_trace
   (FILE	*fp,
    SEGYBREELHEAD *bin_reel_header,
    DATA_HDR	**phdr,
    int		**pdata,
    int		segy_fmt)
{
    DATA_HDR *hdr;
    EXT_TIME et;
    SEGYDREELHEAD char_reel_header;
    SEGYHEAD segyhead;
    double drate;
    int *data = NULL;
    short int *sdata = NULL;
    int data_format, i;
    char *stn;
    
    /* Read the trace header.						*/
    if (fread(&segyhead,sizeof(SEGYHEAD),1,fp)!=1) {
	fprintf(stderr, "Error reading in a SEG-Y Binary reel header.\n");
	exit(1);
    }

    hdr = new_data_hdr();
    hdr->seq_no = 1;

    /* Set fields in DATA_HDR from SEGY headers.			*/
    switch (segy_fmt) {
      case BERKELEY_FORMAT:
	/* Berkeley station is either 8 or 6 characters,		*/
	/* beginning at first additional field or 2 bytes later.	*/
	/* Station is first 3 characters, component is rest.		*/
	stn = segyhead.station_name;
	if (stn[0] == '0' && stn[1] == '0') stn += 2;
	strncpy (hdr->station_id, stn, 3);
	strcpy (hdr->channel_id, stn + 3);
	strcpy (hdr->network_id, network);
	et.year = bin_reel_header->nyear;
	et.doy  = bin_reel_header->nday;
	et.hour = bin_reel_header->nhour;
	et.minute = bin_reel_header->nmin;
	et.second = bin_reel_header->nsec;
	et.usec = bin_reel_header->nmsec * USECS_PER_MSEC;
	dy_to_mdy (et.doy, et.year, &et.month, &et.day);
	hdr->begtime = hdr->hdrtime = ext_to_int(et);
	drate = 1000000.0 / bin_reel_header->sr;
	hdr->sample_rate = (drate >= 1.0) ? 
	    roundoff(drate) : 
	    -1. * roundoff(1.0/drate);
	hdr->sample_rate_mult = 1;
	hdr->num_samples = bin_reel_header->knsamp;
        switch (bin_reel_header->kform) {
	  case 1:   fprintf (stderr, "IBM Floating point format unsupported.\n"); exit(1);
	  case 2:   data_format = INT_32; break;
	  case 3:   data_format = INT_16; break;
	  case 4:   fprintf (stderr, "Gain range format unsupported.\n"); exit(1);
	  default:  fprintf (stderr, "Invald data format code: %d\n", bin_reel_header->kform);
		    exit(1);
	}
	break;

      case PASSCAL_FORMAT:
	/*
	strcpy (hdr->station_id, segyhead.station_name);
	strcpy (hdr->channel_id, segyhead.channel_name);
	*/
	/* Passcal convention uses:					*/
	/*	inst_no         as station_id				*/
	/*	channel_number  as channel_id				*/
	sprintf (hdr->station_id,"%05d",segyhead.inst_no);
	sprintf (hdr->channel_id,"%03d",segyhead.channel_number);
	strcpy (hdr->network_id, network);
	et.year = segyhead.year;
	et.doy = segyhead.day;
	et.hour = segyhead.hour;
	et.minute = segyhead.minute;
	et.second = segyhead.second;
	dy_to_mdy (et.doy, et.year, &et.month, &et.day);
	et.usec = segyhead.m_secs * USECS_PER_MSEC;
	hdr->begtime = hdr->hdrtime = ext_to_int (et);
	drate = (segyhead.deltaSample == 1) ?
	    (double)1000000.0 / segyhead.samp_rate :
	    (double)1000000.0 / segyhead.deltaSample;
	hdr->sample_rate = (drate >= 1) ? 
	    roundoff(drate) : 
	    -1 * roundoff(1.0/drate);
	hdr->sample_rate_mult = 1;
	hdr->num_samples = (segyhead.sampleLength == 32767) ?
	    segyhead.num_samps :
	    segyhead.sampleLength;
	switch (segyhead.data_form) {
	  case 0:   data_format = INT_16; break;
	  case 1:   data_format = INT_32; break;
	  default:  fprintf (stderr, "Invald data_format code: %d\n", segyhead.data_form);
		    exit(1);
	}
	break;

      default:
	fprintf (stderr, "Unknown SEGY format type\n");
	exit (1);
    }

    trim (hdr->station_id);
    trim (hdr->channel_id);
    trim (hdr->network_id);

    /* Perform optional station and channel mapping.	*/
    i = 0;
    if (map) {
	while (map[i].segy_station[0] != '\0') {
	    if (fnmatch(map[i].segy_station, hdr->station_id, 0) == 0 &&
		fnmatch(map[i].segy_channel, hdr->channel_id, 0) == 0) {
		if (strcmp(map[i].seed_station,"*") != 0) 
		    strcpy(hdr->station_id,map[i].seed_station);
		if (strcmp(map[i].seed_channel,"*") != 0) 
		    strcpy(hdr->channel_id,map[i].seed_channel);
		break;
	    }
	    i++;
	}
    }
    
    /*	Read SEGY trace data.						*/
    data = (int *)malloc(hdr->num_samples * sizeof(int));
    if (data == NULL) {
	fprintf (stderr, "Error mallocing data array for %d samples\n", hdr->num_samples);
	exit (1);
    }
    switch (data_format) {
      case INT_32:
	if (fread(data,hdr->num_samples * sizeof(int),1,fp)!=1) {
	    fprintf (stderr, "Error reading %d samples of data\n", hdr->num_samples);
	    exit(1);
	}
	*pdata = data;
	break;
      case INT_16:
	sdata = (short int *)malloc(hdr->num_samples * sizeof(short int));
	if (sdata == NULL) {
	    fprintf (stderr, "Error mallocing data array for %d samples\n", hdr->num_samples);
	    exit (1);
	}
	if (fread(sdata,hdr->num_samples * sizeof(short int),1,fp)!=1) {
	    fprintf (stderr, "Error reading %d samples of data\n", hdr->num_samples);
	    exit(1);
	}
	for (i=0; i<hdr->num_samples; i++) {
	    data[i] = sdata[i];
	}
	free (sdata);
	*pdata = data;
	break;
      default:
	fprintf (stderr, "Unsupported data format: %d\n", data_format);
	exit (1);
    }
    *phdr = hdr;
    return (1);
}
